       PROGRAM MAP_STEEL_COLLAPSE
C     FINITE DIFFERENCE METHOD TO SOLVE THE DIFFUSION PROFILE COLLAPSE
C        COPYRIGHT:  S. S. Babu and H.K.D.H. Bhadeshia
C                    Department of Materials Science and Metallurgy
C                    University of Cambridge
C                    Cambridge  Cb23QZ
C
C Compilated by C.Capdevila Montes. University of Cambridge - CENIM. 
C 		Departamento de Metalurgia Fisica. 
C		Avda. Gregorio del Amo, 8. 
C		Madrid 28040.
C		July/August 1997.
C------------------------------------------------------------------------------
C
C BIBLIOGRAPHY:
C
C	H.K.D.H.Bhadeshia,Metal Science,(1981),Vol. 15, pp.477.
C	H.K.D.H.Bhadeshia,Metal Science,(1980),Vol. 14, pp.477.
C	D.E.Coates, Metallurgical Transactions, (1973), vol.4, pp.395.
C	D.E.Coates, Metallurgical Transactions, (1973), vol.4, pp.1077.
C	D.E.Coates, Metallurgical Transactions, (1973), vol.4, pp.2313.
C	J.Crank,"The mathematics of Diffusion",Clarendon Press,Oxford,(1975).
C	S.S.Babu, PhD. Disertation, University of Cambridge, (1991).
C	S.S.Babu & H.K.D.H.Bhadeshia,Mat. Sci. and Tech.,Vol.6,(1991),pp.1005.
C	S.S.Babu & H.K.D.H.Bhadeshia,Mat. Trans. JIM,Vol.36,(1991),pp.679.
C
C------------------------------------------------------------------------------
C
C VARIABLES:
C
C   1-Inputs:
C
C        Trans. Temperature.          -  CTEMP    in  Deg C
C        Set time at TSP              -  TIME     in  Secs
C        Thickness of Allotriomorphs  -  HALF     in  CM
C        Prior austenite grain size   -  LINEAL   in  CM
C        No of Slices to be considerd -  NSLI
C        C - C interaction Energy     -  W    (from NEWOMEGA) Mole Fraction
C        X gamma/alpha     for TSP    -  XGAG  (from MUCG46)  Mole Fraction
C        X in austenite               -  XBAR
C
C
C------------------------------------------------------------------------------
C
C NOTES:
C	
C Initial Profile is based in Coates , Met. Trans., 1973, vol. 4, pp.395
C
C The following calculations deal with the one-dimensional growth
C of planar grain boundary allotriomorphs from opposite side of grain.
C This process may be considered in two stages. The first involves 
C parabolic growth from both sides according to the earlier exact 
C calculations.
C During the second stage of inmpingment the driving force due to the 
C carbon activity gradient is gradually reduced and the transformation 
C rate correspondingly decrease to a value consistent with a uniform and
C equal carbon chemical potential in both phases. 
C
C Equilibrium concentration at interface is fixed at temperature tested
C
C One-dimensional growth model.
C
C The finite diference method is based on the technique discussed by Crank 
C (1975). The model assumes a one dimensional diffusion process ahead of
C a  planar allotriomorphic - austenite interface. The initial profile 
C set up at time t=0.0, is given by analitical equation calculated by
C Coates (1973). The composition profile is then subjected to finite 
C difference method after normalising the variables to dimensionaless 
C quantities suitable for this method.
C
C 18/08/97
C Subrouitnes RAN,INDEX,RANK:
C Based on Fortran Power Station 4.0 (Rank.for & index.for). 
C Adapted to replace M01DAF NAG fortran Library Routine document.
C C.Capdevila - Montes. 
C University of Cambridge, Dept. Materials Science and Metallurgy,UK
C CENIM - CSIC, Departamento de Metalurgia Fisica, Madrid, Spain
C
C 3/09/97
C Data of the evolution of profile of carbon are stored at OUT file.
C Ready to plot.
C      
      IMPLICIT NONE
      DOUBLE PRECISION C(1000,2),XWORK(1000),X(1000),XN(1000),
     & CN(0:1000,2),YP(1000),CP(1000),CPROF(1000,2),xlest(40)
      DOUBLE PRECISION CM(12),LINEAL,XAGA,XGAG,TI,REFR,XB,XE,
     & CARBON,SI,MN,NI,MO,CR,V,CO,CU,AL,WO,CTEMP,XEQ,CNF,ANS,
     & TIME,TIME2,TEMPISO
	DOUBLE PRECISION DINCR,PI,R,A5,HALF,HH,KK,XBAR,XINCR,
     & D,AREA,TINCR,STEP,T,T10,T20,W,W1,EER
	DOUBLE PRECISION MAP_STEEL_XALPH
      INTEGER I,II,J,NSLI,Z
C Natural constants to work
      PARAMETER (Z=12,PI=3.141592654D+00,R=8.31432D+00)
C
      A5=1.0D+00
      HH=6.6262D-34
      KK=1.38062D-23
      W1=48570.0D+00
      NSLI=220
C
C Reading data regarding chemical composition, half-thickness,
C austenite grain size, temperature and holding time for growing ferrite.
C Austenite grain in cm.
C
      open (7,file='out.dat')
      open (unit=3,file='out')
      read (5,102) CARBON,SI,MN,NI,MO,CR,V,CO,CU,AL,WO
      read (5,*) LINEAL
102   format (11f6.4)
114   FORMAT(F6.2)
      READ(5,*)CTEMP
      T=CTEMP+273.0D+00
      READ(5,*)TIME
      READ(5,*)HALF
      READ(5,*)TIME2
      READ(5,*)TEMPISO
C
C Initial composition.
C
      T=CTEMP+273.0D+00
153   CM(1)=CARBON
      CM(2)=SI
      CM(3)=MN
      CM(4)=NI
      CM(5)=MO
      CM(6)=CR
      CM(7)=V
      CM(8)=CO
      CM(9)=CU
      CM(10)=AL
      CM(11)=WO
      write(*,104)(CM(I),I=1,7)
      write(7,104)(CM(I),I=1,7)
      write(*,121)
      write(7,121)
      WRITE(*,105)CTEMP
      WRITE(*,120)TIME
      WRITE(*,115)LINEAL
      WRITE(7,105)CTEMP
      WRITE(7,120)TIME
      WRITE(7,115)LINEAL
C
C Half diameter of the austenite grain have to be used (cm)
C
      LINEAL=0.5D+00*LINEAL
      LINEAL=LINEAL*1.0D-04
C
C Concentration of carbon in the interface austenite-ferrite in mole
C fraccion (XGAG=XEQ), and the concentration of carbon into ferrite
C (XAGA).
C
148   CALL NEWOMEGA(CM,W,XBAR,T10,T20,0)
      XAGA=MAP_STEEL_XALPH(CTEMP)
      CALL CONCEN(CTEMP,T10,T20,W,XEQ)
      XGAG=XEQ
C
C Initial linear variation of carbon and austneite length
C
      XINCR = (XGAG - XBAR)/NSLI
      C(1,1) = XGAG
      DINCR = (LINEAL - HALF)/NSLI
      DO 10 I  = 1, NSLI
        C(I,1)   = C(1,1) - (I-1) * XINCR
        X(I) = C(I,1)
        XWORK(I) = HALF+ (I-1)* DINCR
10    CONTINUE
C
C Effective diffusivity of carbon into austenite, taking account of the
C fact that this diffusivity is concentration dependent. User Siller a 
C McLellan theory to express the concentration dependence, and Lacher et al
C theory to allow for the effect of substitutional alloying elements on 
C the activity and W of carbon in austenite.
C
      CALL DIFFUS(W,XBAR,CTEMP,XAGA,XGAG,ANS)
      D=ANS
C
C Calculation of the initial profile. Based on Coates, 1973.
C
      CALL PROF(NSLI,HALF,XGAG,XBAR,D,TIME2,XWORK,CPROF)
      WRITE(*,110)
      WRITE(7,110)
C
C Initial profile at the beguinning of the process
C
      DO 2 I=1,NSLI
        C(I,1)=CPROF(I,1)
2     CONTINUE
      C(1,2)=C(1,1)

C
C Normalisation
C
      XINCR = (LINEAL - HALF)/NSLI
      DO 3 I=1,NSLI
        CN(I,1)=C(I,1)/XGAG
        XN(I) = (XWORK(I) - HALF)/(LINEAL-HALF)*100000.0
        XN(I)=INT(XN(I)/10.0)        
3     CONTINUE
C
      DO 50 I = 1,NSLI
            CP(I)=CN(I,1)
50    CONTINUE

C
C Calculation of the area under the curve
C
      CALL TRAPZ(XN,CP,AREA,EER,NSLI)
      WRITE(*,106)AREA
      WRITE(*,121)
      WRITE(7,106)AREA
      WRITE(7,121)
C
C - - - - - - - - - -  Finite difference method  - - - - - - - - - - -
C
C According to:
C
C	S.S.Babu, PhD. Disertation, Appendix Chapter 4.
C
C The coefficient R=D*t/x**2 is taken to be 0.1. This particular value was 
C assumed after checking that variation of R coefficient from 0.1 - 0.3 
C made negligible difference to the results.
C	REFR=R
C
      WRITE(*,107)
      WRITE(7,107)
      TI=0.0D+00
      WRITE(*,111)
      WRITE(*,109)C(1,2),TI
      WRITE(7,111)
      WRITE(7,109)C(1,2),TI
      REFR=0.1D+00
      J=1
      DO WHILE (TI.LE.TIME)
         TINCR=REFR*XINCR*XINCR/D
         TI=TI+TINCR
         CALL FDF(AREA,NSLI,REFR,CN,XN,YP)
         DO 60 II=1,NSLI
           C(II,1)=XGAG*CN(II,1)
           C(II,2)=XGAG*YP(II)
60       CONTINUE
C
C Now the new interface composition
C
         XE=C(1,2)
         XB=C(NSLI,2)
         DO 40 II=1,NSLI
           X(II)=C(II,2)
40       CONTINUE
         CALL DIFFUS(W,XB,TEMPISO,XAGA,XE,D)
         WRITE(*,109)C(1,2),TI
         WRITE(7,109)C(1,2),TI
C
C Profile data. 'OUT' file
C
         if (ti.ge.j) then
                 write(3,1080)ti
                 write(3,1110)
		 do 222 i=1,30
                   xlest(i)=half+(i-1)*((lineal-half)/30)
		   write(3,109)c(i,2),xlest(i)
222              continue
                 step=time/6
        	 j=j+step
         end if
         CNF=AREA/(XN(NSLI)-XN(1))
         IF (CN(1,2).LE.CNF) THEN
              STOP
         END IF
         CALL SWAP(CN,C,NSLI)
      END DO
C~~~~~~~~~~~~~~~~~~~ Format statment only ~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C
1080  format(/,8X,'Profile corresponding at time of ',f7.2,' seconds',/)
1110  format(11x,'INTERFACE CONCENTRATION',5x,'DISTANCE (cm)',/)
1091  format(15x,e14.7,5x,e14.7,5x,e14.7)
1111  format(e14.5,3x,i5)
110   FORMAT(5X,'+++++++Initial Profile Calculation is over+++++++')
111   FORMAT(14X,'INTERFACE CONCENT.',10X,'TIME',/)
109   FORMAT(15X,E14.7,8X,E14.7)
108   FORMAT(10X,E14.7,2X,E14.7,2x,E14.7)
107   FORMAT(//,10X,'/////-Finite difference method starts-/////',//)
106   FORMAT(/,5x,'Area under the normalised curve=',E14.7,/)
104   FORMAT(//,12X,'****Chemical composition****',///
     &,2x,'Carbon=',f6.4
     &,15x,'Silicon=',f6.4,/,2x,'Manganese=',f6.4,12x,'Nickel=',f6.4
     &,/,2x,'Molybdenum=',f6.4,11x,'Cromium=',f6.4,/,2x,'Vanadium='
     &,f6.4,//)
115   FORMAT(5X,'Austenite grain size in microns=',f6.2,/)
105   FORMAT(/,5X,'Isothermal temperature=',f6.1)
121   FORMAT('********************************************************'
     &'*************')
120   format(5x,'Holding time=',f7.2)
C
C~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
C
1     CONTINUE
      WRITE(6,*) 'Finished'
      STOP
      END
C
C**********************************************************************
C
C The finite diference method is based on the technique discussed by Crank C (1975). The model assumes a one dimensional diffusion process ahead of
C a  planar allotriomorphic - austenite interface. The initial profile 
C set up at time t=0.0, is given by analitical equation calculated by
C Coates (1973). The composition profile is then subjected to finite 
C difference method after normalising the variables to dimensionaless 
C quantities suitable for this method.
C S.S. Babu. PhD disetation. University of Cambridge. 1991.
C C.Capdevila - Montes. 
C University of Cambridge, Dept. Materials Science and Metallurgy,UK
C CENIM - CSIC, Departamento de Metalurgia Fisica, Madrid, Spain
C --------------------------------------------------------------------
C      FINITE DIFFERENCE METHOD
C --------------------------------------------------------------------
C
        SUBROUTINE FDF(AREA,NSLI,R,CN,XN,YP)
        IMPLICIT NONE
        INTEGER I,II,NSLI
        DOUBLE PRECISION CN(0:1000,2),YP(1000),XN(1000)
        DOUBLE PRECISION R,AREA,ANS
        DO 10 II=1,NSLI
          I = NSLI - (II-1)*1
          IF (I .EQ. NSLI)  THEN
             CN(I,2)=CN(I,1)+R*(2.0*(CN(I-1,1)-CN(I,1)))
          ELSE
             CN(I,2)=CN(I,1)+R*(CN(I+1,1)-2.0*(CN(I,1))+CN(I-1,1))
          END IF
          IF (I .EQ. 1) THEN
            CN(1,2)  = CN(1,1)
          END IF
10      CONTINUE
        DO 20 I = 1,NSLI
          YP(I) =CN(I,2)
20      CONTINUE
C
C Convergence. Same area
C
        CALL CONV(XN,YP,NSLI,ANS,CN(1,1),CN(2,2),AREA)
        CN(1,2)=YP(1)
        RETURN
        END
C
C**********************************************************************
C-----------------------------------------------------------------------
C    CALL CONV(XN,YP,NSLI,ANS,ERR,IFAIL,CN(1,1),CN(2,2),AREA)
C-----------------------------------------------------------------------
C The purpose of this subroutine is to determine how the composition
C  at the interface changes with time.
C
        SUBROUTINE CONV(X,Y,N,AREA,REF1,REF2,AREF)
        IMPLICIT NONE
        INTEGER IRANK(25)
	  INTEGER ID,IRAN,JCOUNT,N,NDIV
        DOUBLE PRECISION X(N),Y(N),YITER(25),EITER(25)
        DOUBLE PRECISION STEP,DMUL,AREF,REF1,REF2,AREA,CHECK,
     &                   SURF,TEMP 
        step = 1.0E-5
        Y(1) = REF1
        Y(2) = REF2
111     NDIV = 25
        DMUL = (Y(1) - Y(2))/NDIV
C
        IF (DMUL .LE. 1.0E-10) THEN
              Y(1) = (Y(2)+Y(1))/2.0
              Y(2) = REF2
              RETURN
        ENDIF
C
        jcount=0
        DO 10 ID = 1,NDIV
          Y(1) = Y(1) - DMUL
          YITER(ID) = Y(1)
          jcount=jcount+1
          CALL TRAPZ(X,Y,SURF,EITER(id),N)
          CHECK = DABS(AREF-surf)/AREF
          IF (CHECK .LT. step .AND. Y(1) .GE. REF2) THEN
            Y(2) = REF2
            GOTO 1111
          ENDIF
C
10      CONTINUE
        CALL RAN(EITER,IRANK,NDIV)
        IRAN=IRANK(1)
        IF (IRAN .EQ. 1) THEN
        Y(1) = YITER(1)
        Y(2) = YITER(1) - 24.0*DMUL
        ENDIF
        IF (IRAN .EQ. 25) THEN
        Y(1) = YITER(25) + 24.0*DMUL
        Y(2) = YITER(25)
        ENDIF
        IF (IRAN .GE. 2 .AND. IRAN .LE. 24) THEN
        Y(1) = YITER(IRAN+1)
        Y(2) = YITER(IRAN-1)
        IF (Y(1) .LT. Y(2)) THEN
        TEMP = Y(1)
        Y(1) = Y(2)
        Y(2) = TEMP
        ENDIF
        GOTO 111
        ENDIF

1111    RETURN
        END
C**********************************************************************
C----------------------------------------------------------------------
C     SWAP FOR NEXT TIME INTERVAL
C----------------------------------------------------------------------
        SUBROUTINE SWAP(CN,C,NSLI)
        IMPLICIT NONE
        DOUBLE PRECISION C(1000,2),CN(0:1000,2)
        DOUBLE PRECISION TEMP
	  INTEGER I,NSLI
C
        DO 10 I = 1,NSLI
           TEMP = CN(I,2)
           CN(I,1) = TEMP
           TEMP = C(I,2)
           C(I,1) = TEMP
10      CONTINUE
        RETURN
        END
C**********************************************************************
C ---------------------------------------------------------------------
C       COMPOSITION PROFILE
C ---------------------------------------------------------------------
C Based on Coates, Met. Trans., 1973, vol.4, pp.395.
C
        SUBROUTINE PROF(NSLI,AZ,XEQ,XBAR,ADIFF,DELT,XW,CARB)
        IMPLICIT NONE
        DOUBLE PRECISION AB,AK,AZ,ADIFF,DELT,MAP_UTIL_DAERF,
     &                   XBAR,XEQ
        DOUBLE PRECISION CARB(1000),XW(1000)
	  INTEGER I,NSLI
        DO 20 I=1,NSLI
           AK = 1.00-MAP_UTIL_DAERF(XW(i)/2/DSQRT(ADIFF*DELT))
           AB = 1.00-MAP_UTIL_DAERF(AZ/2/DSQRT(ADIFF*DELT))
           IF (AB .LT. 0.01D-45) THEN
                  CARB(I) = XBAR
           ELSE
                  CARB(I) = XBAR + (XEQ - XBAR) * (AK/AB)
           ENDIF
           IF (XW(I) .EQ. AZ) THEN
                CARB(I) =  XEQ
           ENDIF
C          WRITE(6,*) CARB(I),XW(I),AZ
5          FORMAT(1X,4(D12.4))
20      CONTINUE
        RETURN
        END
C
C      
C**********************************************************************
C Subroutine based on MUCG46.FOR.Copyright Dr. H. K. D. H. Bhadeshia,
C University of Cambridge
C Department of Materials Science and Metallurgy, Pembroke St.
C Cambridge CB2 3QZ C Version 1.7 July 1993 CG modified
C Subroutine used to calculate the equilibrium concentration on the
C interface into austenite (XEQ).
C 
      SUBROUTINE CONCEN(CTEMP,T10,T20,W,XEQ)
      IMPLICIT NONE
      DOUBLE PRECISION CTEMP,W,XEQ,AJ,F,TEQ,ETEQ,T,T10,T20,R
      DOUBLE PRECISION MAP_STEEL_AFEG,MAP_STEEL_DAFEG,ENERGY
C
      PARAMETER (R=8.31432D+00)
      T=CTEMP+273.0D+00
      F=ENERGY(T,T10,T20)
      AJ=1.0D+00-DEXP(-W/(R*T))
      XEQ=0.05D+00
C	
51    CONTINUE
      TEQ=R*T*MAP_STEEL_AFEG(XEQ,AJ)-F
C      write(*,*)TEQ,aj,xeq,t
      IF (DABS(TEQ) .LT. 1.0) GOTO 2
      ETEQ=MAP_STEEL_DAFEG(XEQ,AJ)*R*T
      XEQ=XEQ-(TEQ/ETEQ)
      GOTO 51
C
2     CONTINUE
      RETURN
      END
C
C**********************************************************************
C
      SUBROUTINE NEWOMEGA(C,W,XBAR,T10,T20,ID)
C----------------------------------------------------------------------
C SUBROUTINE TO CALCULATE THE CARBON CARBON INTERACTION ENERGY IN
C AUSTENITE, AS A FUNCTION OF ALLOY COMPOSITION.  BASED ON .MUCG18
C THE ANSWER IS IN JOULES PER MOL.   **7 OCTOBER 1981**
C Feed in the alloy concentration in the following order
C  C    SI   MN   NI   MO   CR   V    CO   CU   AL    W      FE
C       In this Program first the Weight % is converted into Mol fraction
C----------------------------------------------------------------------
C
C  1.   Y() contains only the at% of substitutional considering alone
C       T10 and T20 is calculated for the alloy concentration based on
C       Y()
C  2.   Then C-C interaction (W) is calculated based on the ploynomials
C       derived
C       Trial and Error for the interaction energies for Al and Cu
C       MODIFIED FOR OTHER ALLOYING ELEMENTS BY SURESH BABU - C - C
C       APPROX
C       THE CARBON - CARBON INTERACTION MISSING FOR CU AND AL
C----------------------------------------------------------------------
      IMPLICIT NONE
      DOUBLE PRECISION C(12),D(12),P(12),Y(12),B1,B2,B3,
     & T10,T20,W,XBAR
	INTEGER I,ID,IU,NCO
C	          
      B3=0.0D+00
      IF (ID .GE. 2) GOTO 2222
      NCO = 12
      IF (NCO .GT. 12) STOP
C
      D(12)=C(1)+C(2)+C(3)+C(4)+C(5)+C(6)+C(7)+C(8)+C(9)+C(10)+C(11)
      D(12)=100.0D+00-D(12)
      D(12) =D(12)/55.84D+00
      D(1)  =C(1)/12.0115D+00
      D(2)  =C(2)/28.09D+00
      D(3)  =C(3)/54.94D+00
      D(4)  =C(4)/58.71D+00
      D(5)  =C(5)/95.94D+00
      D(6)  =C(6)/52.0D+00
      D(7)  =C(7)/50.94D+00
      D(8)  =C(8)/58.94D+00
      D(9)  =C(9)/63.54D+00
      D(10) =C(10)/26.98D+00
      D(11) =C(11)/183.85D+00
      B1=D(1)+D(2)+D(3)+D(4)+D(5)+D(6)+D(7)+D(8)+D(9)+D(10)+D(11)+D(12)
C------------------------
C------------------------
      DO 106 IU=1,12
        D(IU)=D(IU)/B1
106   CONTINUE
C------------------------
2222  DO 107 IU=2,11
        Y(IU)=D(IU)/D(12)
107   CONTINUE
C
2223  XBAR=D(1)
      XBAR=DINT(10000.0D+00*XBAR)
      XBAR=XBAR/10000
C----------------------------------------------------------------------
C       Delta T_nm and Delta T_mag were taken for Co from
C       Aaronson, Pound and Domain work - Trans.Met.Soc. AIME,
C       Vol.236 May, 1966, p. 769   (S. Babu, 18.7.1989)
C       C    SI   MN   NI   MO   CR   V    CO   CU   AL    W      FE
C       1    2    3    4    5    6    7    8    9    10    11     12

C----------------------------------------------------------------------
      T10=Y(2)*(-3)+Y(3)*2+Y(4)*12+Y(5)*(-9)+Y(6)*(-1)+Y(7)*(-12)+
     &    Y(8)* 3.5 +Y(9)*7+Y(10)*(-7)+Y(11)*(-9)
C
C     T10 coefficients are actually Delt_mag - DelT_non.mag
C     T20 coefficients are actually Delt_mag
      T20=-  3*Y(2)-37.5*Y(3)-6*Y(4) -26*Y(5)-19*Y(6)-44*Y(7)+
     &    19.5*Y(8)- 4.5*Y(9)+8*Y(10)-26*Y(11)
C----------------------------------------------------------------------
      P(2)=2013.0341+763.8167*D(2)+45802.87*D(2)**2-280061.63*D(2)**3
     &+3.864D+06*D(2)**4-2.4233D+07*D(2)**5+6.9547D+07*D(2)**6
      P(3)=2012.067-1764.095*D(3)+6287.52*D(3)**2-21647.96*D(3)**3-
     &2.0119D+06*D(3)**4+3.1716D+07*D(3)**5-1.3885D+08*D(3)**6
      P(4)=2006.8017+2330.2424*D(4)-54915.32*D(4)**2+1.6216D+06*D(4)**3
     &-2.4968D+07*D(4)**4+1.8838D+08*D(4)**5-5.5531D+08*D(4)**6
      P(5)=2006.834-2997.314*D(5)-37906.61*D(5)**2+1.0328D+06*D(5)**3
     &-1.3306D+07*D(5)**4+8.411D+07*D(5)**5-2.0826D+08*D(5)**6
      P(6)=2012.367-9224.2655*D(6)+33657.8*D(6)**2-566827.83*D(6)**3
     &+8.5676D+06*D(6)**4-6.7482D+07*D(6)**5 +2.0837D+08*D(6)**6
      P(7)=2011.9996-6247.9118*D(7)+5411.7566*D(7)**2
     &+250118.1085*D(7)**3-4.1676D+06*D(7)**4
C----------------------------------------------------------------------
C           CO  CU   AL    W      FE
C           8   9    10    11     12
C           EFFECT TAKEN FROM HKDB - MET.SCI - 1981
C           RESULT IN J/MOLE    (S. Babu, 18.7.1989)
C           W SAME AS MOLYBDENUM  IN INTERACION ENERGY
C----------------------------------------------------------------------
      P(8)=(8427.00+5986*D(8))/4.187
      P(9)=  2011.0
      P(10)= 2011.0
      P(11)= 2006.834-2997.314*D(5)-37906.61*D(5)**2+1.0328D+06*D(5)**3
     &-1.3306D+07*D(5)**4+8.411D+07*D(5)**5-2.0826D+08*D(5)**6
      B2=0.0D+00
C
      DO 108 IU=2,11
        B3=B3+P(IU)*Y(IU)
        B2=B2+Y(IU)
108   CONTINUE
C
      IF (B2 .EQ. 0.0D+00) THEN
	      W=8054.0
      ELSE
	      W=(B3/B2)*4.187
      ENDIF
C
      IF (B3 .EQ. 0.0) W = 8054.0
C      
      DO 999 I = 1, 12
        C(I) = D(I)
999   CONTINUE
C
      RETURN
      END
C
C****************************************************************
C     ENERGY ENERGY
      DOUBLE PRECISION FUNCTION ENERGY(T,T10,T20)
	IMPLICIT NONE
      DOUBLE PRECISION T,T10,T20,T7,G,GC
      T7=T-100*T20
      CALL GALGA(G,GC,T7)
      ENERGY=(141.0*T10*4.187) - G                                                                            
      RETURN
      END
C
C ***********************************************************************
C
      SUBROUTINE GALGA (G,GC,T)
C Subroutine to calculate pure Fe's difference in Gibbs Free Energy for
C the reaction ferrite -> austenite, Delta G_o Fe{ferrite to
C austenite}.
C Data due to Kaufman, Clougherty and Weiss, Acta Met. 11 1963 p323.
C Their data used here, with corrections. Range 0-1810K.
C This set of polynomials: R.C.Reed, 30th April 1989,
C written up in RCR10.AABS1.GALGA, with graphs.
C NB Input T in degrees Kelvin, output GC is in J/mol.
C NNB This routine will return a value above the melting point of pure
C iron.
C
      IMPLICIT NONE
	DOUBLE PRECISION G,GC,T
C
      IF (T.LT.3.56D+02) THEN
	G =  0.13132885D+04 + 1.3141025D-02*T - 2.580128186D-03*T*T
      ENDIF

      IF (T.GT.3.56D+02 .AND. T.LT.4.28D+02) THEN
	G =  1.1224052D+03 + 3.542835682*T - 2.2496774D-02*T*T
     &     + 4.326701963D-05*T*T*T - 3.072096339D-08*T*T*T*T
      ENDIF

      IF (T.GE.4.28D+02 .AND. T.LT.6.34D+02) THEN
	G = - 1.7031057D+04 + 0.1448353906D+03*T - 0.428166357*T*T
     &      + 5.50871582D-04*T*T*T - 2.633397102D-07*T*T*T*T
      ENDIF
      IF (T.GE.6.34D+02 .AND. T.LT.7.40D+02) THEN
	G = 0.13022482D+04 + 0.522110231*T - 5.980360202D-03*T*T
     &    + 6.659625695D-06*T*T*T - 2.473169175D-09*T*T*T*T
      ENDIF

      IF (T.GE.7.40D+02 .AND. T.LT.8.60D+02) THEN
	G = - 0.64489093D+05 + 3.095104796D+02*T - 0.547164618*T*T
     &      + 4.255071973D-04*T*T*T - 1.233061536D-07*T*T*T*T
      ENDIF

      IF (T.GE.8.60D+02 .AND. T.LT.9.27D+02) THEN
	G = + 0.13015055D+07 - 5.763825D+03*T + 9.571875*T*T
     &      - 7.0625D-03*T*T*T + 1.953125D-06*T*T*T*T
      ENDIF

      IF (T.GE.9.27D+02 .AND. T.LT.1.08D+03) THEN
	G = + 0.58257237D+05 - 2.229275965D+02*T + 0.322821329*T*T
     &      - 2.089186484D-04*T*T*T + 5.084325376D-08*T*T*T*T
      ENDIF

      IF (T.GE.1.08D+03 .AND. T.LT.1.24D+03) THEN
	G = 0.14043448D+06 - 4.704834516D+02*T + 0.591517384*T*T
     &    - 3.306547594D-04*T*T*T + 6.93139101D-08*T*T*T*T
      ENDIF

      IF (T.GE.1.24D+03 .AND. T.LT.1.502D+03) THEN
	G = 0.15744376D+05 - 4.475440789D+01*T + 4.7873355D-02*T*T
     &    - 2.286184211D-05*T*T*T + 4.111842105D-09*T*T*T*T
      ENDIF

      IF (T.GE.1.502D+03 .AND. T.LT.1.616D+03) THEN
	G = - 0.58637400D+04 + 1.147433313D+01*T
     &      - 7.5500D-03*T*T + 1.6666666D-06*T*T*T
      ENDIF

      IF (T.GE.1.616D+03 .AND. T.LT.1.68D+03) THEN
	G = 0.32972410D+06 - 8.211649405D+02*T + 0.766426516*T*T
     &    - 3.177724674D-04*T*T*T + 4.938936812D-08*T*T*T*T
      ENDIF

      IF (T.GE.1.68D+03) THEN
	G = 0.33271186D+07 - 7.657409945D+03*T
     &    + 6.607976863*T*T - 2.534098538D-03*T*T*T
     &    + 3.643939352D-07*T*T*T*T
      ENDIF

C Convert units from KCW's cal/mol to J/mol:
      G = G * 8.314D+00 / 1.9858D+00
      GC = - 6.41114D4 + 3.2158D1*T
C Carbon: Kirkaldy and Baganis, Met.Trans 9A 495-501 1978.
      RETURN
      END
C**********************************************************************
C
C FUNCTION GIVING THE EQUILIBRIUM MOL.FRAC. CARBON IN ALPHA
C TEMP READ IN AS CENTIGRADE
C BASED ON MY PAPER ON FIRST ORDER QUASICHEMICAL THEROY,METSCI
      DOUBLE PRECISION FUNCTION MAP_STEEL_XALPH(T)
	IMPLICIT NONE
      DOUBLE PRECISION T,CTEMP
      CTEMP=T/900.0D+00
      MAP_STEEL_XALPH=0.1528D-02-0.8816D-02*CTEMP+
     &0.2450D-01*CTEMP**2
     &-0.2417D-01*CTEMP**3+
     &0.6966D-02*CTEMP**4
      RETURN
      END
C**********************************************************************
C
C DAFEG DAFEG DIFFERENTIAL OF LN ACTIVITY OF IRON IN GAMMA
      DOUBLE PRECISION FUNCTION MAP_STEEL_DAFEG(XEQ,AJ)
	IMPLICIT NONE
      DOUBLE PRECISION ETEQ,ETEQ2,DEQ,XEQ,AJ
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      ETEQ=5*((1/(XEQ-1))+2/(1-2*XEQ))
      ETEQ2=6*((4*AJ-1-(0.5/DEQ)*(-2-4*AJ+2*XEQ+16*XEQ*AJ))
     &/(1-2*AJ+(4*AJ-1)*XEQ-DEQ))+6*(4*AJ/(2*AJ*(2*XEQ-1)))
      MAP_STEEL_DAFEG=ETEQ+ETEQ2
      RETURN
      END
C
C*****************************************************************
C AFEG AFEG LN ACTIVITY OF IRON IN GAMMA
      DOUBLE PRECISION FUNCTION MAP_STEEL_AFEG(XEQ,AJ)
	IMPLICIT NONE
      DOUBLE PRECISION XEQ,AJ,DEQ,TEQ
      DEQ=DSQRT(1-2*(1+2*AJ)*XEQ+(1+8*AJ)*XEQ*XEQ)
      TEQ=5*DLOG((1-XEQ)/(1-2*XEQ))
      TEQ=TEQ+6*DLOG(((1-2*AJ+(4*AJ-1)*XEQ-DEQ)/(2*AJ*(2*XEQ-1))))
      MAP_STEEL_AFEG=TEQ
      RETURN
      END
C **********************************************************************
C
C Modified to suppress writing. 5 July 1994
C
C 27 September 1985 same as .CONCDIF:RATE, BUT modified to
C allow easier low T calculations and provide iteration error messages
C TYPICAL DATASET
C 780 0.0100
C 740 0.0176
C 700 0.0266
C 660 0.0389
C 600 0.0552
C Modified 24-6-85 to include XINCR
C - this limits no. of Diff coeffs calculated for T<700C
C PROGRAM MODIFIED, 30-8-83, FOR ACCURATE XAPLPHA
C PROGRAM TO CALCULATE THE EFFECTIVE DIFFUSIVITY OF CARBON IN
C AUSTENITE, TAKING ACCOUNT OF THE FACT THAT THIS DIFFUSIVITY
C IS CONCENTRATION DEPENDENT.  USES SILLER AND MCLELLAN THEORY
C TO EXPRESS THE CONCENTRATION DEPENDENCE, AND LACHER ET AL
C THEORY TO ALLOW FOR THE EFFECT OF SUBSTITUTIONAL ALLOYING ELEMENTS
C ON THE ACTIVITY AND W OF CARBON IN AUSTENITE. **7 OCTOBER 1981**
C TEMPERATURE IS READ IN IN DEGREES CENTIGRADE
C CARBON CONTENT READ IN AS MOL FRAC, EXCEPT FOR THE SUBROUTINE OMEGA,
C WHERE THE CARBON CONTENT AND ALLOYING CONTENT IS READ IN AS WT PCT IN
C THE FIRST LINE OF THE DATASET.
C HH=PLANCKS CONST.JOULES/SEC,KK=BOLTZMANNS CONST.JOULES/DEGREE KELVIN
C     D=DIFFUSIVITY OF CARBON IN AUSTENITE
C     Z=COORDINATION OF INTERSTIAL SITE
C     PSI=COMPOSITION DEPENDENCE OF DIFFUSION COEFFICIENT
C     THETA=NO. C ATOMS/ NO. FE ATOMS
C     ACTIV=ACTIVITY OF CARBON IN AUSTENITE
C     R=GAS CONSTANT
C     X=MOLE FRACTION OF CARBON
C     T=ABSOLUTE TEMPERATURE
C     SIGMA=SITE EXCLUSION PROBABLITY
C     W=CARBON CARBON INTERACTION ENERGY IN AUSTENITE
C
      SUBROUTINE DIFFUS(W,XBAR,CTEMP,XAGA,XGAG,ANS)
      IMPLICIT NONE
C
      DOUBLE PRECISION DIFF(500),CARB(500)
C
	DOUBLE PRECISION HH,KK,A5,R,T,DASH,W,CTEMP,XAGA
	DOUBLE PRECISION ANS,X,XBAR,XINCR,THETA,ACTIV,DACTIV
	DOUBLE PRECISION SIGMA,PSI,ERR,CG,DCG,XGAG
      INTEGER II,II2,II3,Z
C
      PARAMETER (Z=12,R=8.31432D+00)
      HH=6.6262D-34
      KK=1.38062D-23
      A5=1.0D+00      
      T=CTEMP+273.00D+00
      II2=0
      DASH=(KK*T/HH)*DEXP(-(21230.0D+00/T))*DEXP(-31.84D+00)
C
      DO 9   II=1,1000
	CARB(1)=XBAR
	IF (II .LE. 1) GOTO 8
1         IF(((XGAG-XBAR) .LT. 0.005) .OR.
     &			(XGAG-CARB(II-1).LT.0.001)) THEN
C	2nd criterion for IF statement added 13/1/97 to improve
C	accuracy of end test IF (CARB(II).GT.XGAG)
		XINCR=0.0001D+00
      		ELSE
		XINCR=0.001D+00
		ENDIF
4     	CARB(II)=CARB(II-1)+XINCR
      	IF (CARB(II) .GT. XGAG) GOTO 5
8     	X=CARB(II)
      	II2=II2+1
      	THETA=X/(A5-X)
      	ACTIV=CG(X,T,W,R)
      	ACTIV=DEXP(ACTIV)
      	DACTIV=DCG(X,T,W,R)
      	DACTIV=DACTIV*ACTIV
      	DACTIV=DACTIV*A5/((A5+THETA)**2)
      	SIGMA=A5-DEXP((-(W))/(R*T))
      	PSI=ACTIV*(A5+Z*((A5+THETA)/(A5-(A5+Z/2)*THETA+(Z/2)*(A5+Z/2)*
     &	(A5-SIGMA)*THETA*THETA)))+(A5+THETA)*DACTIV
      	DIFF(II)=DASH*PSI
9     CONTINUE

5     II3=0
      CALL TRAPZ(CARB,DIFF,ANS,ERR,II2)
      IF (XGAG .GT. XBAR) THEN
         ANS=ANS/(XGAG-XBAR)
	ELSE 
         ANS=ANS/(XBAR-XGAG)
	ENDIF
	IF (ans .lt. 0.0) ans=0.0
C
c      write(*,*)ans
      RETURN
      END
C
C*****************************************************************
C FUNCTION GIVING LFG  LN(ACTIVITY) OF CARBON IN AUSTENITE
      DOUBLE PRECISION FUNCTION CG(X,T,W,R)
	IMPLICIT NONE
      DOUBLE PRECISION J,DG,DUMMY,T,R,W,X
      J=1-DEXP(-W/(R*T))
      DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
      DUMMY=5*DLOG((1-2*X)/X)+6*W/(R*T)+((38575.0)-(
     &13.48)*T)/(R*T)
      CG=DUMMY+DLOG(((DG-1+3*X)/(DG+1-3*X))**6)
      RETURN
      END
C
C**********************************************************************
C
C FUNCTION GIVING DIFFERENTIAL OF LN(ACTIVITY) OF CARBON IN AUSTENITE, L
C DIFFERENTIAL IS WITH RESPECT TO X
      DOUBLE PRECISION FUNCTION DCG(X,T,W,R)
	IMPLICIT NONE
      DOUBLE PRECISION J,DG,DDG,X,T,W,R
      J=1-DEXP(-W/(R*T))
      DG=DSQRT(1-2*(1+2*J)*X+(1+8*J)*X*X)
      DDG=(0.5/DG)*(-2-4*J+2*X+16*J*X)
      DCG=-((10/(1-2*X))+(5/X))+6*((DDG+3)/(DG-1+3*X)
     &-(DDG-3)/(DG+1-3*X))
      RETURN
      END
C
C**********************************************************************        
C CALCULATE THE AREA UNDER THE CURVE REPRESENTED BY AN ARRAY
C The integral of a function is interpreted as the area under the curve 
C represented by that function. We can note that if we divided the range a<x<b 
C into n equals segments we can approximate the function by a series of 
C straigth lines and we can then easily calculate the required area as the sum
C of the areas of the n-1 trapzoid. 
C
C BIBLIOGRAPHY:
C	T.M.Ellis. 'Fortran 77 Programming'. Ed. Addison-Wesley. 1989.
C	T.M.Ellis, I.R.Philips and T.M.Lahey. 'Fortran 90'.
C				  Ed.  Addison-Wesley. 1990
C Notes:
C
C Based on Fortran Power Station 4.0 (Rank.for). Fortran for PC's.
C C.Capdevila - Montes. 
C University of Cambridge, Dept. Materials Science and Metallurgy,UK
C CENIM - CSIC, Departamento de Metalurgia Fisica, Madrid, Spain
C
      SUBROUTINE TRAPZ(X,Y,INT,ERR,DIM)
	IMPLICIT NONE
      INTEGER DIM,I,MED
	DOUBLE PRECISION X(500),Y(500),AZ(500)
      DOUBLE PRECISION SUM1,A,B,INT,FA,FB,H,FM,C,E,T,
     &                 SUM2,ERR,XM
C
      SUM2=0.D+00
      IF(DIM-1)4,3,1
C
C     INTEGRATION LOOP
    1 DO 2 I=2,DIM
        SUM1=SUM2
        SUM2=SUM2+.5D+00*(X(I)-X(I-1))*(Y(I)+Y(I-1))
        AZ(I-1)=SUM1
    2 CONTINUE
    3 AZ(DIM)=SUM2
      INT=SUM2
C
C Calculate the accuracety in calculation the area
C
      FB=Y(DIM)
      FA=Y(1)
      A=X(1)
      B=X(DIM)
c      print *,'6'
c      write(*,*)a,b,fa,fb
      IF (A.GT.B) THEN 
         H=(A-B)/1.0D+04
         T=H*(FB+FA)/2.0D+00
         XM=B+H/2.0D+00
         MED=NINT(XM)
         FM=Y(MED)
         C=H*(FB+2.0D+00*FM+FA)/4.0D+00
         E=4.0D+00*(C-T)/3.0D+00
c         print *,'7'
      END IF
C
      IF (A.LT.B) THEN
         H=(B-A)/1.0D+04
         T=H*(FB+FA)/2.0D+00
         XM=A+H/2.0D+00
c         write(*,*)h,t,xm
c         print *,'9'
         MED=NINT(XM)
         IF (XM.LE.0.5D+00) MED=1
c         write(*,*)med,h,t
         FM=Y(MED)
         C=H*(FA+2.0D+00*FM+FB)/4.0D+00
         E=4.0D+00*(T-C)/3.0D+00
c         write(*,*)fm,c,e
c         print *,'8'
      END IF
C
C The error is
C
c      print *,'9'
C
      ERR=E
    4 CONTINUE
      RETURN
      END
C**********************************************************************
C
C CONSTRUCT A RANK TABLE FOR AN ARRAY 
C Based on Fortran Power Station 4.0 (Rank.for). Fortran for PC's.
C Adapted to replace M01DAF NAG fortran Library Routine document.
C C.Capdevila - Montes. 
C University of Cambridge, Dept. Materials Science and Metallurgy,UK
C CENIM - CSIC, Departamento de Metalurgia Fisica, Madrid, Spain
C
	SUBROUTINE RAN(DERR,IRANK,NDIM)
	  IMPLICIT NONE
        INTEGER NDIM,INDX(50),IRANK(ndim)
        DOUBLE PRECISION DERR(25)
C        
        CALL INDCX(ndim,DERR,INDX)
        CALL RANKING(ndim,INDX,IRANK)
c        write(*,*) 'original array is:'
c        write(*,*)(derr(j),j=1,100)
c        write(*,*) 'table of ranks is:'
c        write(*,*)(irank(j),j=1,100)
        RETURN
        END
C**********************************************************************
      SUBROUTINE RANKING(n,indx,irank)
	IMPLICIT NONE
      INTEGER J,N,indx(n),irank(n)
C
      do 11 j=1,n
        irank(indx(j))=j
11    continue
      return
      END
C**********************************************************************
C CONSTRUCT AN INDEX FOR AN ARRAY
C Based on Fortran Power Station 4.0 (Rank.for). Fortran for PC's.
C Adapted to replace M01DAF NAG fortran Library Routine document.
C C.Capdevila - Montes. 
C University of Cambridge, Dept. Materials Science and Metallurgy,UK
C CENIM - CSIC, Departamento de Metalurgia Fisica, Madrid, Spain
C 
      SUBROUTINE INDCX(n,arr,indx)
	IMPLICIT NONE
      INTEGER n,indx(n),M,NSTACK
      PARAMETER (M=7,NSTACK=50)
      INTEGER i,indxt,ir,itemp,j,jstack,k,l,istack(NSTACK)
      DOUBLE PRECISION arr(n)
	DOUBLE PRECISION A
C
      do 11 j=1,n
        indx(j)=j
11    continue
C
      jstack=0
      l=1
      ir=n
1     if(ir-l.lt.M)then
        do 13 j=l+1,ir
          indxt=indx(j)
          a=arr(indxt)
          do 12 i=j-1,l,-1
            if(arr(indx(i)).le.a)goto 2
            indx(i+1)=indx(i)
12        continue
          i=l-1
2         indx(i+1)=indxt
13      continue
        if(jstack.eq.0)return
        ir=istack(jstack)
        l=istack(jstack-1)
        jstack=jstack-2
      else
        k=(l+ir)/2
        itemp=indx(k)
        indx(k)=indx(l+1)
        indx(l+1)=itemp
        if(arr(indx(l)).gt.arr(indx(ir)))then
          itemp=indx(l)
          indx(l)=indx(ir)
          indx(ir)=itemp
        endif
        if(arr(indx(l+1)).gt.arr(indx(ir)))then
          itemp=indx(l+1)
          indx(l+1)=indx(ir)
          indx(ir)=itemp
        endif
        if(arr(indx(l)).gt.arr(indx(l+1)))then
          itemp=indx(l)
          indx(l)=indx(l+1)
          indx(l+1)=itemp
        endif
        i=l+1
        j=ir
        indxt=indx(l+1)
        a=arr(indxt)
3       continue
C
          i=i+1
        if(arr(indx(i)).lt.a)goto 3
4       continue
C
          j=j-1
        if(arr(indx(j)).gt.a)goto 4
        if(j.lt.i)goto 5
        itemp=indx(i)
        indx(i)=indx(j)
        indx(j)=itemp
        goto 3
C
5       indx(l+1)=indx(j)
        indx(j)=indxt
        jstack=jstack+2
        if(jstack.gt.NSTACK)pause 'NSTACK too small in indexx'
        if(ir-i+1.ge.j-l)then
          istack(jstack)=ir
          istack(jstack-1)=i
          ir=j-1
        else
          istack(jstack)=j-1
          istack(jstack-1)=l
          l=i
        endif
      endif
      goto 1
      END

C
C**********************************************************************
C
C
C  Error Function
C
      DOUBLE PRECISION FUNCTION MAP_UTIL_DAERF(Y)
      IMPLICIT NONE            
      INTEGER           ISW,I
      DOUBLE PRECISION  P(5),Q(3),P1(8),Q1(7),P2(5),Q2(4)
      DOUBLE PRECISION  Y,XMIN,XLARGE,SSQPI,X,RES,XNUM,XDEN,XSQ,XI
C
C     COEFFICIENTS FOR 0.0 .LE. Y .LT. .477
C
      DATA               P(1)/-.44422647396874/,
     1                   P(2)/10.731707253648/,
     2                   P(3)/15.915606197771/,
     3                   P(4)/374.81624081284/,
     4                   P(5)/2.5612422994823D-02/
      DATA               Q(1)/17.903143558843/,
     1                   Q(2)/124.82892031581/,
     2                   Q(3)/332.17224470532/
C
C             COEFFICIENTS FOR .477 .LE. Y .LE. 4.0
C
      DATA               P1(1)/7.2117582508831/,
     1                   P1(2)/43.162227222057/,
     2                   P1(3)/152.98928504694/,
     3                   P1(4)/339.32081673434/,
     4                   P1(5)/451.91895371187/,
     5                   P1(6)/300.45926102016/,
     6                   P1(7)/-1.3686485738272D-07/,
     7                   P1(8)/.56419551747897/
      DATA               Q1(1)/77.000152935229/,
     1                   Q1(2)/277.58544474399/,
     2                   Q1(3)/638.98026446563/,
     3                   Q1(4)/931.35409485061/,
     4                   Q1(5)/790.95092532790/,
     5                   Q1(6)/300.45926095698/,
     6                   Q1(7)/12.782727319629/
C
C           COEFFICIENTS FOR 4.0 .LT. Y
C
      DATA               P2(1)/-.22695659353969/,
     1                   P2(2)/-4.9473091062325D-02/,
     2                   P2(3)/-2.9961070770354D-03/,
     3                   P2(4)/-2.2319245973418D-02/,
     4                   P2(5)/-2.7866130860965D-01/
      DATA               Q2(1)/1.0516751070679/,
     1                   Q2(2)/.19130892610783/,
     2                   Q2(3)/1.0620923052847D-02/,
     3                   Q2(4)/1.9873320181714/
C
C         CONSTANTS
C
      DATA               XMIN/1.0D-8/,XLARGE/5.6875D0/
      DATA               SSQPI/.56418958354776/
C
C       FIRST EXECUTABLE STATEMENT
C
      X   = Y
      ISW = 1
      IF (X.GE.0.0D0) GO TO 5
      ISW = -1
      X   = -X
C
5     CONTINUE
      IF (X.LT..477D0) GO TO 10
      IF (X.LE.4.0D0) GO TO 25
      IF (X.LT.XLARGE) GO TO 35
      RES = 1.D0
      GO TO 50
C                                  ABS(Y) .LT. .477, EVALUATE
C                                  APPROXIMATION FOR ERF
10    IF (X.LT.XMIN) GO TO 20
      XSQ = X*X
      XNUM = P(5)
      DO 15 I=1,4
         XNUM = XNUM*XSQ+P(I)
15    CONTINUE
C
      XDEN = ((Q(1)+XSQ)*XSQ+Q(2))*XSQ+Q(3)
      RES = X*XNUM/XDEN
      GO TO 50
20    RES = X*P(4)/Q(3)
      GO TO 50
C                                  .477 .LE. ABS(Y) .LE. 4.0
C                                  EVALUATE APPROXIMATION FOR ERF
25    XSQ = X*X
      XNUM = P1(7)*X+P1(8)
      XDEN = X+Q1(7)
C
      DO 30 I=1,6
         XNUM = XNUM*X+P1(I)
         XDEN = XDEN*X+Q1(I)
30    CONTINUE
C
      RES = XNUM/XDEN
      GO TO 45
C
C   4.0 .LT. ABS(Y), EVALUATE
C   APPROXIMATION FOR ERF
C
35    XSQ = X*X
      XI = 1.0D0/XSQ
      XNUM = P2(4)*XI+P2(5)
      XDEN = XI+Q2(4)
C
      DO 40 I=1,3
         XNUM = XNUM*XI+P2(I)
         XDEN = XDEN*XI+Q2(I)
40    CONTINUE
C
      RES = (SSQPI+XI*XNUM/XDEN)/X
45    RES = RES*DEXP(-XSQ)
      RES = 1.0D0-RES
50    IF (ISW.EQ.-1) RES = -RES
      MAP_UTIL_DAERF = RES
C
      RETURN
      END
C